
/*

     Primitives for writing a qcd data file in a format suitable for
     reading by FORTRAN.

*/

#include <stdio.h>
#include <fcntl.h>
#include "fstrmhdr.h"
#include "spnhdr.h"

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

static void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "ERROR: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

FDATA *fdata_open_write (file)
char *file ;
{
  FDATA *result ;

  result = (FDATA *) malloc (sizeof (FDATA)) ;

  if (file == NULL)
    result->desc = 1 ;
  else
  {
    char err_mess [128] ;

    result->desc = open (file, O_CREAT | O_TRUNC | O_WRONLY, 0x1a4) ;

    strcpy (err_mess, "Failed to open \"") ;
    strcat (err_mess, file) ;
    strcat (err_mess, "\"") ;
    sys_assert (result->desc >= 0, err_mess) ;
  }

  return result ;
}

void fdata_close (data)
FDATA *data ;
{
  int result ;

  close (data->desc) ;
  free  (data) ;
}

static void write_buffer (data, len_to_write)
FDATA *data ;
int len_to_write ;
{
  int result ;

  result   = write (data->desc, data->buf, len_to_write) ;
  /*printf ("result = %d\n", result) ;*/
  sys_assert (result == len_to_write, "Write failed") ;
}

void fdata_write_header (data, num_params, params, num_coeffs, coeffs)
FDATA *data ;
int num_params, num_coeffs ;
int params [] ;
float coeffs [] ;
{
  float *buf ;
  int i, ptr ;

  buf = (float *) data->buf ;

  for (i = 0; i < (FBUFFER_SIZE / sizeof (float)); i++)
    buf [i] = 0.0 ;

  ptr = 1 ;

  buf [ptr] = (float) num_params ;
  ptr++ ;

  for (i = 0; i < num_params; i++)
  {
    buf [ptr] = (float) params [i] ;
    ptr++ ;
  }

  buf [ptr] = (float) num_coeffs ;
  ptr++ ;

  for (i = 0; i < num_coeffs; i++)
  {
    buf [ptr] = coeffs [i] ;
    ptr++ ;
  }

  write_buffer (data, FBUFFER_SIZE) ;
}

void fdata_write_field_begin (data)
FDATA *data ;
{
  data->site_size        = sizeof (float) * N_SPIN * N_COL * 2 ;
  data->buf_ptr          = 0 ;
}

void fdata_write_field_record (data, field_record)
FDATA *data ;
float *field_record ;
{
  int cur_rec_space = FBUFFER_SIZE - data->buf_ptr ;
  int residue       = data->site_size - cur_rec_space ;

  if (residue > 0)
  {
    if (cur_rec_space > 0)
      memcpy (data->buf + data->buf_ptr, (char *) field_record, cur_rec_space) ;
    write_buffer (data, FBUFFER_SIZE) ;
    memcpy (data->buf, ((char *) field_record) + cur_rec_space, residue) ;
    data->buf_ptr = residue ;
  }
  else
  {
    memcpy (data->buf + data->buf_ptr, (char *) field_record, data->site_size) ;
    data->buf_ptr += data->site_size ;
  }
}

void fdata_write_field_end   (data)
FDATA *data ;
{
  if (data->buf_ptr > 0)
  {
    write_buffer (data, FBUFFER_SIZE) ;
  }
}

